using System;
using System.Data;
using System.Configuration;
using System.Collections;
using System.Web;
using System.Web.Security;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.UI.WebControls.WebParts;
using System.Web.UI.HtmlControls;

using CMS.GlobalHelper;
using CMS.CMSHelper;
using CMS.SiteProvider;
using CMS.TreeEngine;
using CMS.WorkflowEngine;
using CMS.UIControls;
using CMS.Staging;

using TreeNode = CMS.TreeEngine.TreeNode;
using CMS.SettingsProvider;

public partial class CMSDesk_Content_Properties_Alias_List : CMSDeskPage
{
    #region "Private variables"

    protected int nodeId = 0;
    protected string mSave = null;
    protected string imagesUrl = null;

    TreeNode node = null;
    TreeProvider tree = null;

    #endregion
    protected void Page_Load(object sender, EventArgs e)
    {
        // Register the scripts
        ScriptHelper.RegisterProgress(this.Page);
        
        UIContext.PropertyTab = PropertyTabEnum.URLs;

        nodeId = ValidationHelper.GetInteger(Request.QueryString["nodeid"], 0);
        imagesUrl = ResolveUrl(UrlHelper.ImagesDirectory + "/CMSDesk/Content/");

        if (nodeId > 0)
        {
            // Get the node
            tree = new TreeProvider(CMSContext.CurrentUser);
            node = tree.SelectSingleNode(nodeId);

            if (node != null)
            {
                if (node.NodeAliasPath == "/")
                {
                    this.valAlias.Visible = false;
                }

                // Check read permissions
                if (CMSContext.CurrentUser.IsAuthorizedPerTreeNode(node, NodePermissionsEnum.Read) == AuthorizationResultEnum.Denied)
                {
                    RedirectToAccessDenied(String.Format(ResHelper.GetString("cmsdesk.notauthorizedtoreaddocument"), node.NodeAliasPath));
                }
                // Check modify permissions
                else if (CMSContext.CurrentUser.IsAuthorizedPerTreeNode(node, NodePermissionsEnum.Modify) == AuthorizationResultEnum.Denied)
                {
                    UniGridAlias.GridView.Enabled = false;
                    this.lnkNewAlias.Enabled = false;
                    this.lblInfo.Visible = true;
                    this.lblInfo.Text = String.Format(ResHelper.GetString("cmsdesk.notauthorizedtoeditdocument"), node.NodeAliasPath);                    
                    
                    // Disable save button
                    this.lnkSave.Enabled = false;
                    this.lnkSave.CssClass = "MenuItemEditDisabled";
                    this.imgSave.ImageUrl = imagesUrl + "savedisabled.gif";             
                }
                else
                {
                    this.lblInfo.Visible = false;
                    ltlScript.Text = ScriptHelper.GetScript("var node = " + nodeId.ToString() + "; \n var deleteMsg = '" + ResHelper.GetString("DocumentAlias.DeleteMsg") + "';");

                    UniGridAlias.OnAction += new OnActionEventHandler(UniGridAlias_OnAction);
                    UniGridAlias.OnExternalDataBound += new OnExternalDataBoundEventHandler(UniGridAlias_OnExternalDataBound);

                    this.imgSave.ImageUrl = imagesUrl + "save.gif";

                    // Shortcuts script
                    this.Page.ClientScript.RegisterClientScriptBlock(typeof(string), "EditShortcuts",
                            "<script type=\"text/javascript\" src=\"" + ResolveUrl("~/CMSScripts/shortcuts.js") + "\"></script>" +
                            ScriptHelper.GetScript("function SaveDocument() { " + this.ClientScript.GetPostBackEventReference(this.lnkSave, null) + " } \n"
                        ));
                }                

                UniGridAlias.GridView.GridLines = GridLines.Horizontal;
                UniGridAlias.WhereCondition = "AliasNodeId ='" + nodeId + "'";

                lnkNewAlias.Text = ResHelper.GetString("doc.urls.addnewalias");
                lnkNewAlias.NavigateUrl = "Alias_Edit.aspx?nodeid=" + nodeId.ToString();
                imgNewAlias.ImageUrl = imagesUrl + "Properties/adddocumentalias.gif";

                mSave = ResHelper.GetString("general.save");
                this.lblAlias.Text = ResHelper.GetString("GeneralProperties.Alias");
                this.lblUrlPath.Text = ResHelper.GetString("GeneralProperties.UrlPath");
                this.chkCustomUrl.Text = ResHelper.GetString("GeneralProperties.UseCustomUrlPath");
                this.chkCustomExtensions.Text = ResHelper.GetString("GeneralProperties.UseCustomExtensions");
                this.valAlias.ErrorMessage = ResHelper.GetString("GeneralProperties.RequiresAlias");

                lblExtensions.Text = ResHelper.GetString("doc.urls.urlextensions") + ResHelper.Colon;
                lblCampaign.Text = ResHelper.GetString("doc.urls.trackcampaign") + ResHelper.Colon;

                this.pnlPath.GroupingText = ResHelper.GetString("GeneralProperties.PathGroup");
                this.pnlDocumentAlias.GroupingText = ResHelper.GetString("doc.urls.documentalias");
                this.pnlExtended.GroupingText = ResHelper.GetString("doc.urls.extendedproperties");
                   

                if (!RequestHelper.IsPostBack())
                {
                    ReloadData();
                }
            }
        }
    }


    object UniGridAlias_OnExternalDataBound(object sender, string sourceName, object parameter)
    {
        switch (sourceName.ToLower())
        {
            case "culture":
                return UniGridFunctions.CultureDisplayName(parameter);
        }

        return parameter;
    }


    void UniGridAlias_OnAction(string actionName, object actionArgument)
    {
        // Edit action
        if (DataHelper.GetNotEmpty(actionName, String.Empty) == "edit")
        {
            UrlHelper.Redirect("Alias_Edit.aspx?nodeid=" + nodeId + "&aliasid=" + Convert.ToString(actionArgument));
        }
        // Delete action
        else if (DataHelper.GetNotEmpty(actionName, String.Empty) == "delete")
        {
            int aliasId = ValidationHelper.GetInteger(actionArgument, 0);
            if (aliasId > 0)
            {
                // Delete
                DocumentAliasInfoProvider.DeleteDocumentAliasInfo(aliasId);

                // Log synchronization
                DocumentHelper.LogSynchronization(node, TaskTypeEnum.UpdateDocument, tree, SynchronizationInfoProvider.ENABLED_SERVERS);
            }
        }
    }


    protected void chkCustomUrl_CheckedChanged(object sender, EventArgs e)
    {
        this.txtUrlPath.Enabled = this.chkCustomUrl.Checked;
        if (!this.chkCustomUrl.Checked)
        {
            txtUrlPath.Text = "";
        }
    }


    protected void chkCustomExtensions_CheckedChanged(object sender, EventArgs e)
    {
        this.txtExtensions.Enabled = this.chkCustomExtensions.Checked;
        if (!this.chkCustomExtensions.Checked)
        {
            txtExtensions.Text = node.DocumentExtensions;
        }
    }


    protected void lnkSave_Click(object sender, EventArgs e)
    {
        // Get the document
        TreeProvider tree = new TreeProvider(CMSContext.CurrentUser);
        node = tree.SelectSingleNode(nodeId, CMSContext.PreferredCultureCode);
        if (node != null)
        {
            node.NodeAlias = this.txtAlias.Text.Trim();

            string docUrlPath = this.txtUrlPath.Text.Trim();

            if (docUrlPath != "")
            {
                docUrlPath = "/" + docUrlPath.TrimStart('/');
            }

            node.DocumentUrlPath = docUrlPath;
            node.DocumentCampaign = this.txtCampaign.Text;
            
            node.DocumentUseNamePathForUrlPath = !this.chkCustomUrl.Checked;
            if (node.DocumentUseNamePathForUrlPath)
            {
                string urlPath = TreePathUtils.GetUrlPathFromNamePath(node.DocumentNamePath, node.NodeLevel);
                node.DocumentUrlPath = urlPath;
                this.txtUrlPath.Text = urlPath;
            }

            node.DocumentUseCustomExtensions = this.chkCustomExtensions.Checked;
            if (node.DocumentUseCustomExtensions)
            {
                node.DocumentExtensions = txtExtensions.Text;
            }

            // Save the data
            node.Update();

            // Update search index
            if ((node.IsInPublishStep) && (SearchIndexInfoProvider.SearchEnabled))
            {
                SearchTaskInfoProvider.CreateTask(SearchTaskTypeEnum.Update, PredefinedObjectType.DOCUMENT, SearchHelper.ID_FIELD, node.GetSearchID());
            }

            this.txtAlias.Text = node.NodeAlias;
            this.txtUrlPath.Text = node.DocumentUrlPath;

            // Log synchronization
            DocumentHelper.LogSynchronization(node, CMS.Staging.TaskTypeEnum.UpdateDocument, tree);

            this.lblInfo.Visible = true;
            this.lblInfo.Text = ResHelper.GetString("General.ChangesSaved");

            this.UniGridAlias.ReloadData();
        }
    }


    private void ReloadData()
    {
        if (node != null)
        {
            // Check read permissions
            if (CMSContext.CurrentUser.IsAuthorizedPerTreeNode(node, NodePermissionsEnum.Read) == AuthorizationResultEnum.Denied)
            {
                RedirectToAccessDenied(String.Format(ResHelper.GetString("cmsdesk.notauthorizedtoreaddocument"), node.NodeAliasPath));
            }
            else
            {
                // Check modify permissions
                if (CMSContext.CurrentUser.IsAuthorizedPerTreeNode(node, NodePermissionsEnum.Modify) == AuthorizationResultEnum.Denied)
                {
                    // show access denied message
                    lblInfo.Text = String.Format(ResHelper.GetString("cmsdesk.notauthorizedtoeditdocument"), node.NodeAliasPath);
                    this.pnlPath.Enabled = false;
                }


                this.chkCustomUrl.Checked = !node.DocumentUseNamePathForUrlPath;
                this.chkCustomExtensions.Checked = node.DocumentUseCustomExtensions;
                this.txtExtensions.Text = node.DocumentExtensions;
                this.txtCampaign.Text = node.DocumentCampaign;
                this.txtAlias.Text = node.NodeAlias;
                this.txtUrlPath.Text = node.DocumentUrlPath;

                if (!this.chkCustomUrl.Checked)
                {
                    txtUrlPath.Enabled = false;
                }

                if (!this.chkCustomExtensions.Checked)
                {
                    txtExtensions.Enabled = false;
                }

                if (node.NodeClassName.ToLower() == "cms.root")
                {
                    this.txtAlias.Enabled = false;
                    this.txtUrlPath.Enabled = false;
                    this.txtUrlPath.Text = "";
                    this.chkCustomUrl.Enabled = false;
                    this.valAlias.Enabled = false;
                }

                if (node.IsLink)
                {
                    this.txtUrlPath.Enabled = false;
                    this.chkCustomUrl.Enabled = false;
                }
            }
        }
    }
}
