using System;
using System.Data;
using System.Web.UI.WebControls;

using CMS.GlobalHelper;
using CMS.CMSHelper;
using CMS.TreeEngine;
using CMS.SiteProvider;
using CMS.UIControls;

public partial class CMSFormControls_CMSPages_RoleSelection : CMSLiveModalPage
{
    #region "Variables"

    protected int groupId = 0;
    protected int siteId = 0;
    protected int nodeId = 0;
    protected bool showOnlyUsers = false;
    protected bool allowMultipleUserSelection = false;
    protected bool useFriendly = false;
    protected CurrentUserInfo currentUser = null;

    #endregion


    protected void Page_Load(Object sender, EventArgs e)
    {
        currentUser = CMSContext.CurrentUser;

        groupId = QueryHelper.GetInteger("groupId", 0);
        siteId = CMSContext.CurrentSite.SiteID;
        useFriendly = QueryHelper.GetBoolean("UseFriendly", false);
        
        string onlyUsers = QueryHelper.GetText("mode", "");
        if (onlyUsers == "user")
        {
            showOnlyUsers = true;
        }
        else if (onlyUsers == "usermultiple")
        {
            showOnlyUsers = true;
            allowMultipleUserSelection = true;
        }

        btnSearch.Text = ResHelper.GetString("general.search");
        btnOK.Text = ResHelper.GetString("general.ok");
        btnCancel.Text = ResHelper.GetString("general.cancel");
        lblUserName.Text = (showOnlyUsers ? ResHelper.GetString("SecurityAddOperator.UserName") : ResHelper.GetString("SecurityAddOperator.RoleName"));
        SetTitleElem();
    }


    /// <summary>
    /// Search button handler
    /// </summary>
    protected void btnSearch_Click(Object sender, EventArgs e)
    {
        DataSet ds = null;
        string searchPhrase = "%" + txtSearch.Text + "%";
        string where = null;

        if (showOnlyUsers)
        {
            if (!currentUser.IsGlobalAdministrator)
            {
                where = "(Hidden = 0 OR Hidden IS NULL)";
            }

            // Find all users whose name contains specified string
            ds = RoleInfoProvider.FindUsersAndRolesForSite(siteId, searchPhrase, true, false, where);
            lstItems.SelectionMode = allowMultipleUserSelection ? ListSelectionMode.Multiple : ListSelectionMode.Single;
        }
        else
        {
            // Find all roles whose name contains specified string
            string whereCondition = (groupId > 0) ? "(GroupID = " + groupId + ")" : "(GroupID IS NULL)";
            
            if (useFriendly)
            {
                whereCondition += "AND RoleName != '" + RoleInfoProvider.AUTHENTICATED + "' AND RoleName != '" + RoleInfoProvider.NOTAUTHENTICATED + "'";
            }
            
            ds = RoleInfoProvider.FindUsersAndRolesForSite(siteId, searchPhrase, false, true, whereCondition);
            lstItems.SelectionMode = ListSelectionMode.Multiple;
        }

        lstItems.DataSource = ds.Tables[0];
        lstItems.DataTextField = "OperatorName";
        lstItems.DataValueField = "OperatorID";
        lstItems.DataBind();
    }


    /// <summary>
    /// OK button handler.
    /// </summary>
    protected void btnOK_Click(Object sender, EventArgs e)
    {
        string mText = String.Empty;
        string mText2 = String.Empty;

        string Id = QueryHelper.GetText("elemId", String.Empty);
        string hiddenId = QueryHelper.GetText("hiddenId", String.Empty);

        // get IDs of the selected users/roles and save changes
        foreach (ListItem item in lstItems.Items)
        {
            if (item.Selected)
            {
                if (item.Value.ToLower().StartsWith("r"))
                {
                    item.Value = item.Value.ToLower().Replace("r", "");
                }
                else if (item.Value.ToLower().StartsWith("u"))
                {
                    item.Value = item.Value.ToLower().Replace("u", "");
                }

                if (showOnlyUsers)
                {
                    UserInfo ui = UserInfoProvider.GetUserInfo(ValidationHelper.GetInteger(item.Value, 0));
                    if (ui != null)
                    {
                        if ((!allowMultipleUserSelection) && (mText.Trim() == ""))
                        {
                            mText = ui.FullName;
                        }
                        else
                        {
                            mText += ";" + ui.UserName;
                        }
                    }
                    if (!allowMultipleUserSelection)
                    {
                        if (ui != null)
                        {
                            ltlScript.Text +=
                                ScriptHelper.GetScript(" CloseAndRefreshUser(" + ScriptHelper.GetString(mText) + ", " + ScriptHelper.GetString(Id) +
                                                       ", '" + ui.UserID + "', " + ScriptHelper.GetString(hiddenId) + "); ");
                        }
                        return;
                    }
                }
                else
                {
                    RoleInfo ri = RoleInfoProvider.GetRoleInfo(ValidationHelper.GetInteger(item.Value, 0));
                    if (ri != null)
                    {
                        if (useFriendly)
                        {
                            mText += ri.DisplayName + ";";
                            mText2 += ri.RoleName + ";";
                        }
                        else
                        {
                            if (mText.Trim() == "")
                            {
                                mText = ri.RoleName;
                            }
                            else
                            {
                                mText = mText + ";" + ri.RoleName;
                            }
                        }
                    }
                }
            }
        }

        if (showOnlyUsers)
        {
            if (allowMultipleUserSelection)
            {
                ltlScript.Text += ScriptHelper.GetScript(" CloseAndRefreshUser(" + ScriptHelper.GetString(mText.Trim(';')) + ", " + ScriptHelper.GetString(Id) + ", " + ScriptHelper.GetString(mText.Trim(';')) + ", " + ScriptHelper.GetString(hiddenId) + "); ");
            }
            else
            {
                // Nothing was selected if we are here
                ltlScript.Text += ScriptHelper.GetScript(" Close();");
            }
        }
        else
        {
            if (useFriendly)
            {
                ltlScript.Text += ScriptHelper.GetScript(" CloseAndRefreshUser(" + ScriptHelper.GetString(mText.Trim(';')) + ", " + ScriptHelper.GetString(Id) + ", " + ScriptHelper.GetString(mText2.Trim(';')) + ", " + ScriptHelper.GetString(hiddenId) + "); ");
            }
            else
            {
                ltlScript.Text += ScriptHelper.GetScript(" CloseAndRefreshRole(" + ScriptHelper.GetString(mText) + ", " + ScriptHelper.GetString(Id) + "); ");
            }
        }
    }


    /// <summary>
    /// Sets the page title
    /// </summary>
    private void SetTitleElem()
    {
        if (showOnlyUsers)
        {
            Page.Title = "Select user";
            titleElem.TitleText = ResHelper.GetString("SecurityAddOperator.OnlyUsersHeader");
        }
        else
        {
            Page.Title = "Select role";
            titleElem.TitleText = ResHelper.GetString("SecurityAddOperator.OnlyRolesHeader");
        }
        titleElem.TitleImage = UrlHelper.ImagesDirectory + "/CMSSiteManager/Administration/Roles/selectuser.gif";
    }


    /// <summary>
    /// Cancel button handler.
    /// </summary>
    protected void btnCancel_Click(Object sender, EventArgs e)
    {
        ltlScript.Text = ScriptHelper.GetScript("Close();");
    }
}


